@@ -11,12 +11,17 @@ import re
 import sys
 import traceback
 from functools import lru_cache
-from typing import Dict, Pattern
+from typing import Dict, Pattern, Tuple
 
 import babel
-import entrypoints
 from packaging.version import parse as parse_version
 
+# See compatibility note on `group` keyword in https://docs.python.org/3/library/importlib.metadata.html#entry-points
+if sys.version_info < (3, 10):
+    from importlib_metadata import entry_points
+else:
+    from importlib.metadata import entry_points
+
 # Entry points
 JUPYTERLAB_LANGUAGEPACK_ENTRY = "jupyterlab.languagepack"
 JUPYTERLAB_LOCALE_ENTRY = "jupyterlab.locale"
@@ -27,6 +32,7 @@ LOCALE_DIR = "locale"
 LC_MESSAGES_DIR = "LC_MESSAGES"
 DEFAULT_DOMAIN = "jupyterlab"
 L10N_SCHEMA_NAME = "@jupyterlab/translation-extension:plugin"
+PY37_OR_LOWER = sys.version_info[:2] <= (3, 7)
 
 _default_schema_context = "schema"
 _default_settings_context = "settings"
@@ -41,9 +47,9 @@ DEFAULT_SCHEMA_SELECTORS = {
     "title": _default_schema_context,
     "description": _default_schema_context,
     # JupyterLab-specific
-    "jupyter\.lab\.setting-icon-label": _default_settings_context,
-    "jupyter\.lab\.menus/.*/label": "menu",
-    "jupyter\.lab\.toolbars/.*/label": "toolbar",
+    r"jupyter\.lab\.setting-icon-label": _default_settings_context,
+    r"jupyter\.lab\.menus/.*/label": "menu",
+    r"jupyter\.lab\.toolbars/.*/label": "toolbar",
 }
 
 
@@ -85,7 +91,7 @@ def _get_installed_language_pack_locales():
     """
     data = {}
     messages = []
-    for entry_point in entrypoints.get_group_all(JUPYTERLAB_LANGUAGEPACK_ENTRY):
+    for entry_point in entry_points(group=JUPYTERLAB_LANGUAGEPACK_ENTRY):
         try:
             data[entry_point.name] = os.path.dirname(entry_point.load().__file__)
         except Exception:
@@ -114,7 +120,7 @@ def _get_installed_package_locales():
     """
     data = {}
     messages = []
-    for entry_point in entrypoints.get_group_all(JUPYTERLAB_LOCALE_ENTRY):
+    for entry_point in entry_points(group=JUPYTERLAB_LOCALE_ENTRY):
         try:
             data[entry_point.name] = os.path.dirname(entry_point.load().__file__)
         except Exception:
@@ -200,9 +206,7 @@ def get_display_name(locale: str, display_locale: str = DEFAULT_LOCALE) -> str:
         Localized `locale` and capitalized language name using `display_locale` as language.
     """
     locale = locale if is_valid_locale(locale) else DEFAULT_LOCALE
-    display_locale = (
-        display_locale if is_valid_locale(display_locale) else DEFAULT_LOCALE
-    )
+    display_locale = display_locale if is_valid_locale(display_locale) else DEFAULT_LOCALE
     loc = babel.Locale.parse(locale)
     display_name = loc.get_display_name(display_locale)
     if display_name:
@@ -247,15 +251,17 @@ def merge_locale_data(language_pack_locale_data, package_locale_data):
     return result
 
 
-def get_installed_packages_locale(locale: str) -> dict:
+def get_installed_packages_locale(locale: str) -> Tuple[dict, str]:
     """
     Get all jupyterlab extensions installed that contain locale data.
 
     Returns
     -------
-    dict
-        Ordered list of available language packs.
-        >>> {"package-name": locale_data, ...}
+    tuple
+        A tuple in the form `(locale_data_dict, message)`,
+        where the `locale_data_dict` is an ordered list
+        of available language packs:
+            >>> {"package-name": locale_data, ...}
 
     Examples
     --------
@@ -284,11 +290,11 @@ def get_installed_packages_locale(locale: str) -> dict:
                     locale_path,
                     locales[locale.lower()],
                     LC_MESSAGES_DIR,
-                    "{0}.json".format(package_name),
+                    f"{package_name}.json",
                 )
                 if os.path.isfile(locale_json_path):
                     try:
-                        with open(locale_json_path, "r", encoding="utf-8") as fh:
+                        with open(locale_json_path, encoding="utf-8") as fh:
                             packages_locale_data[package_name] = json.load(fh)
                     except Exception:
                         messages.append(traceback.format_exc())
@@ -298,7 +304,7 @@ def get_installed_packages_locale(locale: str) -> dict:
 
 # --- API
 # ----------------------------------------------------------------------------
-def get_language_packs(display_locale: str = DEFAULT_LOCALE) -> tuple:
+def get_language_packs(display_locale: str = DEFAULT_LOCALE) -> Tuple[dict, str]:
     """
     Return the available language packs installed in the system.
 
@@ -328,9 +334,7 @@ def get_language_packs(display_locale: str = DEFAULT_LOCALE) -> tuple:
             else:
                 invalid_locales.append(locale)
 
-        display_locale = (
-            display_locale if display_locale in valid_locales else DEFAULT_LOCALE
-        )
+        display_locale = display_locale if display_locale in valid_locales else DEFAULT_LOCALE
         locales = {
             DEFAULT_LOCALE: {
                 "displayName": get_display_name(DEFAULT_LOCALE, display_locale),
@@ -344,9 +348,7 @@ def get_language_packs(display_locale: str = DEFAULT_LOCALE) -> tuple:
             }
 
         if invalid_locales:
-            messages.append(
-                "The following locales are invalid: {0}!".format(invalid_locales)
-            )
+            messages.append(f"The following locales are invalid: {invalid_locales}!")
 
     return locales, "\n".join(messages)
 
@@ -380,7 +382,7 @@ def get_language_pack(locale: str) -> tuple:
                         pkg_name = name.replace(".json", "")
                         json_path = os.path.join(root, name)
                         try:
-                            with open(json_path, "r", encoding="utf-8") as fh:
+                            with open(json_path, encoding="utf-8") as fh:
                                 merged_data = json.load(fh)
                         except Exception:
                             messages.append(traceback.format_exc())
@@ -487,7 +489,14 @@ class TranslationBundle:
         str
             The translated string.
         """
-        return gettext.dpgettext(self._domain, msgctxt, msgid)
+        # Python 3.7 or lower does not offer translations based on context.
+        # On these versions `pgettext` falls back to `gettext`
+        if PY37_OR_LOWER:
+            translation = gettext.dgettext(self._domain, msgid)
+        else:
+            translation = gettext.dpgettext(self._domain, msgctxt, msgid)
+
+        return translation
 
     def npgettext(self, msgctxt: str, msgid: str, msgid_plural: str, n: int) -> str:
         """
@@ -509,7 +518,14 @@ class TranslationBundle:
         str
             The translated string.
         """
-        return gettext.dnpgettext(self._domain, msgctxt, msgid, msgid_plural, n)
+        # Python 3.7 or lower does not offer translations based on context.
+        # On these versions `npgettext` falls back to `ngettext`
+        if PY37_OR_LOWER:
+            translation = gettext.dngettext(self._domain, msgid, msgid_plural, n)
+        else:
+            translation = gettext.dnpgettext(self._domain, msgctxt, msgid, msgid_plural, n)
+
+        return translation
 
     # Shorthands
     def __(self, msgid: str) -> str:
@@ -566,7 +582,7 @@ class TranslationBundle:
         """
         return self.pgettext(msgctxt, msgid)
 
-    def _np(self, msgctxt: str, msgid: str, msgid_plural: str, n: str) -> str:
+    def _np(self, msgctxt: str, msgid: str, msgid_plural: str, n: int) -> str:
         """
         Shorthand for npgettext.
 
@@ -643,7 +659,7 @@ class translator:
         if is_valid_locale(locale):
             cls._LOCALE = locale
             translator._update_env(locale)
-            for domain, bundle in cls._TRANSLATORS.items():
+            for _, bundle in cls._TRANSLATORS.items():
                 bundle.update_locale(locale)
 
     @classmethod
@@ -688,7 +704,7 @@ class translator:
 
             if isinstance(value, str):
                 matched = False
-                for pattern, context in to_translate.items():
+                for pattern, context in to_translate.items():  # noqa
                     if pattern.fullmatch(path):
                         matched = True
                         break
@@ -715,7 +731,7 @@ class translator:
     @staticmethod
     def translate_schema(schema: Dict) -> Dict:
         """Translate a schema.
-        
+
         Parameters
         ----------
         schema: dict
